from datetime import date

import click
from pyfiglet import Figlet

from promissory_note.commands import IssuePromissoryNoteCommand
from promissory_note.gateways.services import PillowImageGenerationService, SendGridEmailPromissoryNoteIssued
from promissory_note.use_cases import IssuePromissoryNote


def print_blue(text, begin='', *args, **kwargs):
    print(f'\033[34m{begin}{text}\033[0m', *args, **kwargs)


def print_green(text, begin='', *args, **kwargs):
    print(f'\033[32m{begin}{text}\033[0m', *args, **kwargs)


def print_application_name():
    font = Figlet(font='pepper')
    application_name = font.renderText('Promissory Note')
    print_blue(application_name)


def print_description():
    description = 'Welcome, send your promissory notes'
    print_blue(description, end='\n\n')


@click.command()
@click.option('--number',
              default=1,
              prompt=True,
              type=click.INT)
@click.option('--due-date',
              default=date.today().strftime('%d/%m/%Y'),
              prompt=True,
              type=click.DateTime(('%d/%m/%Y',)))
@click.option('--value',
              default=100.99,
              prompt=True,
              type=click.FLOAT)
@click.option('--beneficiary-name',
              default='Steve',
              prompt=True,
              type=click.STRING)
@click.option('--beneficiary-cpf',
              default='11111111199',
              prompt=True,
              type=click.STRING)
@click.option('--beneficiary-email',
              default='captain@marvel.com',
              prompt=True,
              type=click.STRING)
@click.option('--currency',
              default='dollar',
              prompt=True,
              type=click.STRING)
@click.option('--city-payment',
              default='New York',
              prompt=True,
              type=click.STRING)
@click.option('--state-payment',
              default='New York',
              prompt=True,
              type=click.STRING)
@click.option('--emitter-name',
              default='Tony Stark',
              prompt=True,
              type=click.STRING)
@click.option('--emitter-cpf',
              default='11111111188',
              prompt=True,
              type=click.STRING)
@click.option('--emitter-address',
              default='New York',
              prompt=True,
              type=click.STRING)
@click.option('--emitter-email',
              default='iron_man@marveil.com',
              prompt=True,
              type=click.STRING)
@click.option('--issuance-date',
              default=date.today().strftime('%d/%m/%Y'),
              prompt=True,
              type=click.DateTime(('%d/%m/%Y',)))
def issue_promissory_note(number,
                          due_date,
                          value,
                          beneficiary_name,
                          beneficiary_cpf,
                          beneficiary_email,
                          currency,
                          city_payment,
                          state_payment,
                          emitter_name,
                          emitter_cpf,
                          emitter_address,
                          emitter_email,
                          issuance_date):
    Cli().issued_promissory_note(number,
                                 due_date,
                                 value,
                                 beneficiary_name,
                                 beneficiary_cpf,
                                 beneficiary_email,
                                 currency,
                                 city_payment,
                                 state_payment,
                                 emitter_name,
                                 emitter_cpf,
                                 emitter_address,
                                 emitter_email,
                                 issuance_date)


class Cli:
    def __init__(self):
        self._image_generation_service = PillowImageGenerationService()
        self._email_service = SendGridEmailPromissoryNoteIssued()

        self._issue_promissory_note = IssuePromissoryNote(self._image_generation_service, self._email_service)

        self._number = None
        self._due_date = None
        self._value = None
        self._beneficiary_name = None
        self._beneficiary_cpf = None
        self._beneficiary_email = None
        self._currency = None
        self._city_payment = None
        self._state_payment = None
        self._emitter_name = None
        self._emitter_cpf = None
        self._emitter_address = None
        self._emitter_email = None
        self._issuance_date = None

        self._issue_promissory_note_command = None

    @property
    def number(self):
        return self._number

    @property
    def due_date(self):
        return self._due_date.date()

    @property
    def value(self):
        return self._value

    @property
    def beneficiary_name(self):
        return self._beneficiary_name

    @property
    def beneficiary_cpf(self):
        return self._beneficiary_cpf

    @property
    def beneficiary_email(self):
        return self._beneficiary_email

    @property
    def currency(self):
        return self._currency

    @property
    def city_payment(self):
        return self._city_payment

    @property
    def state_payment(self):
        return self._state_payment

    @property
    def emitter_name(self):
        return self._emitter_name

    @property
    def emitter_cpf(self):
        return self._emitter_cpf

    @property
    def emitter_address(self):
        return self._emitter_address

    @property
    def emitter_email(self):
        return self._emitter_email

    @property
    def issuance_date(self):
        return self._issuance_date.date()

    def issued_promissory_note(self,
                               number,
                               due_date,
                               value,
                               beneficiary_name,
                               beneficiary_cpf,
                               beneficiary_email,
                               currency,
                               city_payment,
                               state_payment,
                               emitter_name,
                               emitter_cpf,
                               emitter_address,
                               emitter_email,
                               issuance_date):
        self._number = number
        self._due_date = due_date
        self._value = value
        self._beneficiary_name = beneficiary_name
        self._beneficiary_cpf = beneficiary_cpf
        self._beneficiary_email = beneficiary_email
        self._currency = currency
        self._city_payment = city_payment
        self._state_payment = state_payment
        self._emitter_name = emitter_name
        self._emitter_cpf = emitter_cpf
        self._emitter_address = emitter_address
        self._emitter_email = emitter_email
        self._issuance_date = issuance_date

        self._create_command()

        self._issue_promissory_note.execute(command=self._issue_promissory_note_command)

        print_green('Emission request sent !!!', begin='\n\n')

    def _create_command(self):
        self._issue_promissory_note_command = IssuePromissoryNoteCommand(number=self.number,
                                                                         due_date=self.due_date,
                                                                         value=self.value,
                                                                         beneficiary_name=self.beneficiary_name,
                                                                         beneficiary_cpf=self.beneficiary_cpf,
                                                                         beneficiary_email=self.beneficiary_email,
                                                                         currency=self.currency,
                                                                         city_payment=self.city_payment,
                                                                         state_payment=self.state_payment,
                                                                         emitter_name=self.emitter_name,
                                                                         emitter_cpf=self.emitter_cpf,
                                                                         emitter_address=self.emitter_address,
                                                                         emitter_email=self.emitter_email,
                                                                         issuance_date=self.issuance_date)


def main():
    print_application_name()
    print_description()
    issue_promissory_note()


if __name__ == '__main__':
    main()



import unittest
from datetime import date, datetime
from os import chdir, listdir, path, remove
from unittest.mock import patch, Mock

from pyfiglet import Figlet

from promissory_note.gateways.cli import Cli, print_green, print_blue, print_application_name, print_description, main
from promissory_note.gateways.services import CONTENT_DIR, PROMISSORY_NOTE_IMAGE, OPEN_SANS


class PrintBlueTests(unittest.TestCase):
    @patch('builtins.print')
    def test_should_print_text_when_print_green_is_called_with_text(self, print):
        print_blue('Tony', begin='\n')

        print.assert_called_with('\033[34m\nTony\033[0m')


class PrintGreenTests(unittest.TestCase):
    @patch('builtins.print')
    def test_should_print_text_when_print_green_is_called_with_text(self, print):
        print_green('Tony', begin='\n')

        print.assert_called_with('\033[32m\nTony\033[0m')


class PrintApplicationNameTests(unittest.TestCase):
    @patch('promissory_note.gateways.cli.print_blue')
    def test_should_print_application_name_when_is_called(self, print_blue):
        font = Figlet(font='pepper')
        application_name = font.renderText('Promissory Note')

        print_application_name()

        print_blue.assert_called_with(application_name)


class PrintDescriptionTests(unittest.TestCase):
    @patch('promissory_note.gateways.cli.print_blue')
    def test_should_print_description_when_is_called(self, print_blue):
        description = 'Welcome, send your promissory notes'

        print_description()

        print_blue.assert_called_with(description, end='\n\n')


class CliTests(unittest.TestCase):
    def setUp(self):
        self._cli = Cli()

        self._cli._image_generation_service = Mock()
        self._cli._email_service = Mock()
        self._cli._issue_promissory_note = Mock()

        self._execute_issued_promissory_note()
        self._remove_created_images()

    def tearDown(self):
        self._remove_created_images()

    def test_should_cli_print_promissory_note_issued_success(self):
        self._cli._issue_promissory_note.execute.assert_called_once()

    def test_should_cli_contains_number(self):
        self.assertEqual(1, self._cli.number)

    def test_should_cli_contains_due_date(self):
        self.assertEqual(date(year=2020, month=12, day=12), self._cli.due_date)

    def test_should_cli_contains_value(self):
        self.assertEqual(100.99, self._cli.value)

    def test_should_cli_contains_beneficiary_name(self):
        self.assertEqual('Steve', self._cli.beneficiary_name)

    def test_should_cli_contains_beneficiary_cpf(self):
        self.assertEqual('11111111199', self._cli.beneficiary_cpf)

    def test_should_cli_contains_beneficiary_email(self):
        self.assertEqual('captain@marvel.com', self._cli.beneficiary_email)

    def test_should_cli_contains_currency(self):
        self.assertEqual('real', self._cli.currency)

    def test_should_cli_contains_city_payment(self):
        self.assertEqual('New York', self._cli.city_payment)

    def test_should_cli_contains_state_payment(self):
        self.assertEqual('New York', self._cli.state_payment)

    def test_should_cli_contains_emitter_name(self):
        self.assertEqual('Tony Stark', self._cli.emitter_name)

    def test_should_cli_contains_emitter_cpf(self):
        self.assertEqual('11111111188', self._cli.emitter_cpf)

    def test_should_cli_contains_emitter_address(self):
        self.assertEqual('New York', self._cli.emitter_address)

    def test_should_cli_contains_emitter_email(self):
        self.assertEqual('iron_man@marvel.com', self._cli.emitter_email)

    def test_should_cli_contains_issuance_date(self):
        self.assertEqual(date(year=2020, month=12, day=12), self._cli.issuance_date)

    def _execute_issued_promissory_note(self):
        self._cli.issued_promissory_note(number=1,
                                         due_date=datetime(day=12, month=12, year=2020),
                                         value=100.99,
                                         beneficiary_name='Steve',
                                         beneficiary_cpf='11111111199',
                                         beneficiary_email='captain@marvel.com',
                                         currency='real',
                                         city_payment='New York',
                                         state_payment='New York',
                                         emitter_name='Tony Stark',
                                         emitter_cpf='11111111188',
                                         emitter_address='New York',
                                         emitter_email='iron_man@marvel.com',
                                         issuance_date=datetime(day=12, month=12, year=2020))

    def _remove_created_images(self):
        chdir(CONTENT_DIR)

        created_images = [file for file in listdir(CONTENT_DIR) if
                          file not in (path.basename(PROMISSORY_NOTE_IMAGE), path.basename(OPEN_SANS))]

        for image in created_images:
            remove(image)


class MainTests(unittest.TestCase):
    @patch('promissory_note.gateways.cli.print_application_name')
    @patch('promissory_note.gateways.cli.print_description')
    @patch('promissory_note.gateways.cli.issue_promissory_note')
    def test_should_main_called_print_application_name(self,
                                                       print_application_name,
                                                       print_description,
                                                       issue_promissory_note):
        main()

        print_application_name.assert_called_once()

    @patch('promissory_note.gateways.cli.print_application_name')
    @patch('promissory_note.gateways.cli.print_description')
    @patch('promissory_note.gateways.cli.issue_promissory_note')
    def test_should_main_called_print_description(self,
                                                  print_application_name,
                                                  print_description,
                                                  issue_promissory_note):
        main()

        print_description.assert_called_once()

    @patch('promissory_note.gateways.cli.print_application_name')
    @patch('promissory_note.gateways.cli.print_description')
    @patch('promissory_note.gateways.cli.issue_promissory_note')
    def test_should_main_called_issue_promissory_note(self,
                                                      print_application_name,
                                                      print_description,
                                                      issue_promissory_note):
        main()

        issue_promissory_note.assert_called_once()



import unittest
from datetime import date

from promissory_note.events import PromissoryNoteIssued, PromissoryNoteNotIssued


class PromissoryNoteIssuedTests(unittest.TestCase):
    def setUp(self):
        self._promissory_note_issued = PromissoryNoteIssued(number=1,
                                                            due_date=date.today(),
                                                            value=100.99,
                                                            beneficiary_name='Steve',
                                                            beneficiary_cpf='11111111199',
                                                            beneficiary_email='captain@marvel.com',
                                                            currency='real',
                                                            city_payment='New York',
                                                            state_payment='New York',
                                                            emitter_name='Tony Stark',
                                                            emitter_cpf='11111111188',
                                                            emitter_address='New York',
                                                            emitter_email='iron_man@marvel.com',
                                                            issuance_date=date.today())

    def test_should_promissory_note_issued_contains_number(self):
        self.assertEqual(1, self._promissory_note_issued.number)

    def test_should_promissory_note_issued_contains_due_date(self):
        self.assertEqual(date.today(), self._promissory_note_issued.due_date)

    def test_should_promissory_note_issued_contains_value(self):
        self.assertEqual(100.99, self._promissory_note_issued.value)

    def test_should_promissory_note_issued_contains_beneficiary_name(self):
        self.assertEqual('Steve', self._promissory_note_issued.beneficiary_name)

    def test_should_promissory_note_issued_contains_beneficiary_cpf(self):
        self.assertEqual('11111111199', self._promissory_note_issued.beneficiary_cpf)

    def test_should_promissory_note_issued_contains_beneficiary_email(self):
        self.assertEqual('captain@marvel.com', self._promissory_note_issued.beneficiary_email)

    def test_should_promissory_note_issued_contains_currency(self):
        self.assertEqual('real', self._promissory_note_issued.currency)

    def test_should_promissory_note_issued_contains_city_payment(self):
        self.assertEqual('New York', self._promissory_note_issued.city_payment)

    def test_should_promissory_note_issued_contains_state_payment(self):
        self.assertEqual('New York', self._promissory_note_issued.state_payment)

    def test_should_promissory_note_issued_contains_emitter_name(self):
        self.assertEqual('Tony Stark', self._promissory_note_issued.emitter_name)

    def test_should_promissory_note_issued_contains_emitter_cpf(self):
        self.assertEqual('11111111188', self._promissory_note_issued.emitter_cpf)

    def test_should_promissory_note_issued_contains_emitter_address(self):
        self.assertEqual('New York', self._promissory_note_issued.emitter_address)

    def test_should_promissory_note_issued_contains_emitter_email(self):
        self.assertEqual('iron_man@marvel.com', self._promissory_note_issued.emitter_email)

    def test_should_promissory_note_issued_contains_issuance_date(self):
        self.assertEqual(date.today(), self._promissory_note_issued.issuance_date)

    def test_should_promissory_note_issued_contains_notifications(self):
        self.assertListEqual([], self._promissory_note_issued.notifications)


class PromissoryNoteNotIssuedTests(unittest.TestCase):
    def test_should_promissory_note_not_issued_contains_notifications(self):
        promissory_note_not_issued = PromissoryNoteNotIssued(notifications=[])

        self.assertListEqual([], promissory_note_not_issued.notifications)



import unittest

from promissory_note.value_objects import Name, Cpf, Email


class NameTests(unittest.TestCase):
    def test_should_is_valid_true_when_name_is_not_empty_or_white_space(self):
        name = Name('Steve')

        self.assertTrue(name.is_valid)

    def test_should_is_valid_false_when_name_is_empty(self):
        name = Name('')

        self.assertFalse(name.is_valid)

    def test_should_is_valid_false_when_name_is_none(self):
        name = Name(None)

        self.assertFalse(name.is_valid)

    def test_should_notification_must_contain_invalid_name_message_when_name_is_invalid(self):
        name = Name('')

        notification = name.notifications[0]

        self.assertEqual('invalid name', notification.message)

    def test_should_str_return_name_string(self):
        name = Name('Steve')

        self.assertEqual('Steve', str(name))


class CpfTests(unittest.TestCase):
    def test_should_is_valid_true_when_length_cpf_is_equal_to_eleven(self):
        cpf = Cpf('11111111111')

        self.assertTrue(cpf.is_valid)

    def test_should_is_valid_false_when_length_cpf_is_not_equal_to_eleven(self):
        cpf = Cpf('1111111111')

        self.assertFalse(cpf.is_valid)

    def test_should_notification_must_contain_invalid_cpf_message_when_cpf_is_invalid(self):
        cpf = Cpf('1111111111')

        notification = cpf.notifications[0]

        self.assertEqual('invalid cpf', notification.message)

    def test_should_str_return_cpf_string(self):
        cpf = Cpf('11111111111')

        self.assertEqual('11111111111', str(cpf))


class EmailTests(unittest.TestCase):
    def test_should_is_valid_true_when_email_is_valid(self):
        email = Email('steve@marvel.com')

        self.assertTrue(email.is_valid)

    def test_should_is_valid_false_when_email_is_invalid(self):
        email = Email('anything')

        self.assertFalse(email.is_valid)

    def test_should_notification_must_contain_invalid_email_message_when_email_is_invalid(self):
        email = Email('anything')

        notification = email.notifications[0]

        self.assertEqual('invalid email', notification.message)

    def test_should_str_return_email_string(self):
        email = Email('captain@marvel.com')

        self.assertEqual('captain@marvel.com', str(email))



import base64
import locale
import uuid
from os import getenv
from os.path import dirname, join

from PIL import Image, ImageDraw, ImageFont
from sendgrid import Mail, SendGridAPIClient, Attachment, FileContent, FileType, FileName, Disposition, ContentId

from promissory_note.events import PromissoryNoteIssued

locale.setlocale(locale.LC_ALL, 'pt_BR.UTF-8')

ROOT_DIR = dirname(dirname(__file__))

CONTENT_DIR = join(ROOT_DIR, 'content')

PROMISSORY_NOTE_IMAGE = join(CONTENT_DIR, 'promissory_note.jpg')

OPEN_SANS = join(CONTENT_DIR, 'open_sans.ttf')

TEXT_FONT = ImageFont.truetype(OPEN_SANS, size=16)

TEXT_COLOR = 'rgb(0,0,0)'

NUMBER_POSITION = 200, 34

DUE_DATE_DAY_POSITION = 348, 34

DUE_DATE_MONTH_POSITION = 410, 34

DUE_DATE_YEAR_POSITION = 550, 34

LONG_DUE_DATE_POSITION = 205, 62

VALUE_POSITION = 650, 34

FULL_VALUE_POSITION = 370, 148

BENEFICIARY_NAME_POSITION = 180, 118

BENEFICIARY_NAME_CPF_POSITION = 615, 118

CITY_AND_STATE_PAYMENT_POSITION = 425, 210

EMITTER_NAME_POSITION = 225, 238

ISSUANCE_DATE_DAY_POSITION = 650, 238

ISSUANCE_DATE_MONTH_POSITION = 695, 238

ISSUANCE_DATE_YEAR_POSITION = 735, 238

EMITTER_CPF_POSITION = 225, 264

EMITTER_ADDRESS_POSITION = 525, 264

SEND_GRID_KEY = getenv('SEND_GRID_KEY', 'YOUR_SEND_GRID_KEY')

FROM_EMAIL = getenv('FROM_EMAIL', 'YOUR_FROM_EMAIL')

SUBJECT = 'Promissory note'

CONTENT = '<p>Promissory note issued with success !!!</p>'

FILE_TYPE = 'image/jpeg'

DISPOSITION = 'attachment'


def write(fn):
    def wrapper(self, *args, **kwargs):
        fn(self, *args, **kwargs)
        self._write()

    return wrapper


class PillowImageGenerationService:
    def __init__(self):
        self._image = None
        self._draw = None
        self._position = None
        self._text = None
        self._filename = None
        self._promissory_note_issued = None

    def __call__(self, event):
        if isinstance(event, PromissoryNoteIssued):
            self.generate(event)

    def generate(self, promissory_note_issued):
        self._promissory_note_issued = promissory_note_issued

        self._open_image()

        self._write_number()
        self._write_due_date_day()
        self._write_due_date_month()
        self._write_due_date_year()
        self._write_long_due_date()
        self._write_value()
        self._write_full_value()
        self._write_beneficiary_name()
        self._write_beneficiary_cpf()
        self._write_city_and_state_payment()
        self._write_emitter_name()
        self._write_issuance_date_day()
        self._write_issuance_date_month()
        self._write_issuance_date_year()
        self._write_emitter_cpf()
        self._write_emitter_address()

        self._create_filename()

        self._save_image()

    def _open_image(self):
        self._image = Image.open(PROMISSORY_NOTE_IMAGE)
        self._draw = ImageDraw.Draw(self._image)

    @write
    def _write_number(self):
        self._position = NUMBER_POSITION
        self._text = str(self._promissory_note_issued.number)

    @write
    def _write_due_date_day(self):
        self._position = DUE_DATE_DAY_POSITION
        self._text = self._promissory_note_issued.due_date.strftime('%d')

    @write
    def _write_due_date_month(self):
        self._position = DUE_DATE_MONTH_POSITION
        self._text = self._promissory_note_issued.due_date.strftime('%B')

    @write
    def _write_due_date_year(self):
        self._position = DUE_DATE_YEAR_POSITION
        self._text = self._promissory_note_issued.due_date.strftime('%Y')

    @write
    def _write_long_due_date(self):
        self._position = LONG_DUE_DATE_POSITION
        self._text = self._promissory_note_issued.due_date.strftime('%d de %B de %Y')

    @write
    def _write_value(self):
        self._position = VALUE_POSITION
        self._text = f'{self._promissory_note_issued.value:.2f}'

    @write
    def _write_full_value(self):
        self._position = FULL_VALUE_POSITION
        self._text = f'R$ {self._promissory_note_issued.value:.2f}'

    @write
    def _write_beneficiary_name(self):
        self._position = BENEFICIARY_NAME_POSITION
        self._text = self._promissory_note_issued.beneficiary_name

    @write
    def _write_beneficiary_cpf(self):
        self._position = BENEFICIARY_NAME_CPF_POSITION
        self._text = self._promissory_note_issued.beneficiary_cpf

    @write
    def _write_city_and_state_payment(self):
        self._position = CITY_AND_STATE_PAYMENT_POSITION
        self._text = f'{self._promissory_note_issued.city_payment} - {self._promissory_note_issued.state_payment}'

    @write
    def _write_emitter_name(self):
        self._position = EMITTER_NAME_POSITION
        self._text = self._promissory_note_issued.emitter_name

    @write
    def _write_issuance_date_day(self):
        self._position = ISSUANCE_DATE_DAY_POSITION
        self._text = self._promissory_note_issued.issuance_date.strftime('%d')

    @write
    def _write_issuance_date_month(self):
        self._position = ISSUANCE_DATE_MONTH_POSITION
        self._text = self._promissory_note_issued.issuance_date.strftime('%m')

    @write
    def _write_issuance_date_year(self):
        self._position = ISSUANCE_DATE_YEAR_POSITION
        self._text = self._promissory_note_issued.issuance_date.strftime('%y')

    @write
    def _write_emitter_cpf(self):
        self._position = EMITTER_CPF_POSITION
        self._text = self._promissory_note_issued.emitter_cpf

    @write
    def _write_emitter_address(self):
        self._position = EMITTER_ADDRESS_POSITION
        self._text = self._promissory_note_issued.emitter_address

    def _create_filename(self):
        basename = f'{self._promissory_note_issued.number}_promissory_note.jpg'
        self._filename = join(CONTENT_DIR, basename)

    def _save_image(self):
        self._image.save(self._filename)

    def _write(self):
        self._draw.text(self._position, self._text, fill=TEXT_COLOR, font=TEXT_FONT)


class SendGridEmailPromissoryNoteIssued:
    def __init__(self):
        self._send_grid = SendGridAPIClient(SEND_GRID_KEY)

        self._promissory_note_issued = None

        self._from_email = None
        self._from_to_emails = None
        self._subject = None
        self._content = None
        self._attachment = None

        self._mail = None

    def __call__(self, event):
        if isinstance(event, PromissoryNoteIssued):
            self.send_promissory_note_issued(promissory_note_issued=event)

    def send_promissory_note_issued(self, promissory_note_issued):
        self._promissory_note_issued = promissory_note_issued

        self._create_mail()

        self._send_grid.send(self._mail)

    def _create_mail(self):
        self._create_from_email()
        self._create_to_emails()
        self._create_subject()
        self._create_content()
        self._create_attachment()

        self._mail = Mail(
            from_email=self._from_email,
            to_emails=self._to_emails,
            subject=self._subject,
            html_content=self._content,
        )

        self._mail.attachment = self._attachment

    def _create_from_email(self):
        self._from_email = FROM_EMAIL

    def _create_to_emails(self):
        self._to_emails = [
            self._promissory_note_issued.beneficiary_email,
            self._promissory_note_issued.emitter_email
        ]

    def _create_subject(self):
        self._subject = SUBJECT

    def _create_content(self):
        self._content = CONTENT

    def _create_attachment(self):
        promissory_note_image_name = f'{self._promissory_note_issued.number}_promissory_note.jpg'

        with open(join(CONTENT_DIR, promissory_note_image_name), 'rb') as img:
            data = img.read()

        encoded = base64.b64encode(data).decode()

        file_content = FileContent(encoded)

        file_type = FileType(FILE_TYPE)

        file_name = FileName(promissory_note_image_name)

        disposition = Disposition(DISPOSITION)

        content_id = ContentId(str(uuid.uuid4()))

        attachment = Attachment(file_content=file_content,
                                file_name=file_name,
                                file_type=file_type,
                                disposition=disposition,
                                content_id=content_id)

        self._attachment = attachment



from setuptools import setup, find_packages

setup(
    name='promissory_note',
    version='0.0.5',
    description='Simple promissory note issuance',
    long_description='Simple promissory note issuance',
    url='https://github.com/flaviogf/promissory_note',
    author='Flavio Fernandes',
    author_email='flavio.fernandes6@gmail.com',
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        'pyflunt==1.0.0',
        'pillow==6.0.0',
        'click==7.0',
        'pyfiglet==0.8.post1',
        'sendgrid==6.0.4',
    ],
    entry_points={
        'console_scripts': ['promissory-note-cli=promissory_note.gateways.cli:main'],
    },
    zip_safe=False,
)



import unittest
from datetime import date
from os import chdir, listdir, remove, path
from unittest.mock import Mock

from promissory_note.events import PromissoryNoteIssued, PromissoryNoteNotIssued
from promissory_note.gateways.services import (PillowImageGenerationService,
                                               SendGridEmailPromissoryNoteIssued,
                                               CONTENT_DIR,
                                               PROMISSORY_NOTE_IMAGE,
                                               OPEN_SANS)


class PillowImageGenerationServiceTests(unittest.TestCase):
    def setUp(self):
        self._remove_created_images()

        self._image_generation_service = PillowImageGenerationService()

    def tearDown(self):
        self._remove_created_images()

    def test_should_pillow_image_generation_service_implements_call(self):
        self.assertTrue(callable(self._image_generation_service))

    def test_should_pillow_image_generation_create_image_in_content_dir_when_generate_is_called_with_promissory_note_issued(
            self):
        promissory_note_issued = PromissoryNoteIssued(number=1,
                                                      due_date=date.today(),
                                                      value=100.99,
                                                      beneficiary_name='Steve',
                                                      beneficiary_cpf='11111111199',
                                                      beneficiary_email='captain@marvel.com',
                                                      currency='real',
                                                      city_payment='New York',
                                                      state_payment='New York',
                                                      emitter_name='Tony Stark',
                                                      emitter_cpf='11111111188',
                                                      emitter_address='New York',
                                                      emitter_email='iron_man@marvel.com',
                                                      issuance_date=date.today())

        self._image_generation_service.generate(promissory_note_issued=promissory_note_issued)

        filename = f'{promissory_note_issued.number}_promissory_note.jpg'

        self.assertTrue(filename in listdir(CONTENT_DIR))

    def test_should_pillow_image_generation_create_image_in_content_dir_when_call_is_called_with_promissory_note_issued(
            self):
        promissory_note_issued = PromissoryNoteIssued(number=1,
                                                      due_date=date.today(),
                                                      value=100.99,
                                                      beneficiary_name='Steve',
                                                      beneficiary_cpf='11111111199',
                                                      beneficiary_email='captain@marvel.com',
                                                      currency='real',
                                                      city_payment='New York',
                                                      state_payment='New York',
                                                      emitter_name='Tony Stark',
                                                      emitter_cpf='11111111188',
                                                      emitter_address='New York',
                                                      emitter_email='iron_man@marvel.com',
                                                      issuance_date=date.today())

        self._image_generation_service(event=promissory_note_issued)

        filename = f'{promissory_note_issued.number}_promissory_note.jpg'

        self.assertTrue(filename in listdir(CONTENT_DIR))

    def test_should_pillow_image_generation_dont_create_image_in_content_dir_when__call_is_called_with_promissory_note_not_issued(
            self):
        created_images = [file for file in listdir(CONTENT_DIR) if
                          file not in (path.basename(PROMISSORY_NOTE_IMAGE), path.basename(OPEN_SANS))]

        promissory_note_not_issued = PromissoryNoteNotIssued(notifications=[])

        self._image_generation_service(event=promissory_note_not_issued)

        self.assertListEqual([], created_images)

    def _remove_created_images(self):
        chdir(CONTENT_DIR)

        created_images = [file for file in listdir(CONTENT_DIR) if
                          file not in (path.basename(PROMISSORY_NOTE_IMAGE), path.basename(OPEN_SANS))]

        for image in created_images:
            remove(image)


class SendGridEmailPromissoryNoteIssuedTests(unittest.TestCase):
    def setUp(self):
        self._remove_created_images()

        self._email_service = SendGridEmailPromissoryNoteIssued()
        self._email_service._send_grid = Mock()

    def tearDown(self):
        self._remove_created_images()

    def test_should_send_grid_email_service_is_a_callable(self):
        self.assertTrue(callable(self._email_service))

    def test_should_callable_call_send_promissory_note_issued(self):
        promissory_note_issued = PromissoryNoteIssued(number=1,
                                                      due_date=date.today(),
                                                      value=100.99,
                                                      beneficiary_name='Flavio',
                                                      beneficiary_cpf='11111111199',
                                                      beneficiary_email='flavio.fernandes6@gmail.com',
                                                      currency='real',
                                                      city_payment='New York',
                                                      state_payment='New York',
                                                      emitter_name='Fernando',
                                                      emitter_cpf='11111111188',
                                                      emitter_address='New York',
                                                      emitter_email='nando.franca@yahoo.com',
                                                      issuance_date=date.today())

        image_generation_service = PillowImageGenerationService()

        image_generation_service.generate(promissory_note_issued=promissory_note_issued)

        self._email_service(event=promissory_note_issued)

        self._email_service._send_grid.send.assert_called_once()

    def test_should_send_promissory_note_issued_call_send_grid_api(self):
        promissory_note_issued = PromissoryNoteIssued(number=1,
                                                      due_date=date.today(),
                                                      value=100.99,
                                                      beneficiary_name='Flavio',
                                                      beneficiary_cpf='11111111199',
                                                      beneficiary_email='flavio.fernandes6@gmail.com',
                                                      currency='real',
                                                      city_payment='New York',
                                                      state_payment='New York',
                                                      emitter_name='Fernando',
                                                      emitter_cpf='11111111188',
                                                      emitter_address='New York',
                                                      emitter_email='nando.franca@yahoo.com',
                                                      issuance_date=date.today())

        image_generation_service = PillowImageGenerationService()

        image_generation_service.generate(promissory_note_issued=promissory_note_issued)

        self._email_service.send_promissory_note_issued(promissory_note_issued=promissory_note_issued)

        self._email_service._send_grid.send.assert_called_once()

    def _remove_created_images(self):
        chdir(CONTENT_DIR)

        created_images = [file for file in listdir(CONTENT_DIR) if
                          file not in (path.basename(PROMISSORY_NOTE_IMAGE), path.basename(OPEN_SANS))]

        for image in created_images:
            remove(image)
